package com.xjrsoft.common.utils;


import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.xjrsoft.common.exception.MyException;
import com.xjrsoft.module.organization.dto.WeChatDepartDto;
import com.xjrsoft.module.organization.dto.WeChatUserDto;
import lombok.Data;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;

@Component
@Data
public class WeChatUtil {
    @Value("${xjrsoft.enterpriseWeChat.corpid}")
    public String appKey;

    @Value("${xjrsoft.enterpriseWeChat.secret}")
    public String appSecret;

    @Value("${xjrsoft.enterpriseWeChat.secret1}")
    public String appSecret1;

    @Value("${xjrsoft.enterpriseWeChat.agentid}")
    public Integer agentid;

    @Value("${xjrsoft.mpWeChat.appKey}")
    public String mpAppKey;

    @Value("${xjrsoft.mpWeChat.appSecret}")
    public String mpAppSecret;

    @Value("${xjrsoft.appletWeChat.appKey}")
    public String appletAppKey;

    @Value("${xjrsoft.appletWeChat.appSecret}")
    public String appletAppSecret;

    //公众号获取Openid
    public  String[] getMpOpenid(String code) {
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("appid", mpAppKey);
        paramMap.put("secret", mpAppSecret);
        paramMap.put("code", code);
        paramMap.put("grant_type", "authorization_code");
        String result = HttpUtil.get("https://api.weixin.qq.com/sns/oauth2/access_token", paramMap);
        JSONObject jsonObject = JSONObject.parseObject(result);
        if (jsonObject.containsKey("errcode")) {
            //throw new MyException(result);
            return null;
        }
        String openId = jsonObject.get("openid").toString();
        String unionid = "";
        if (jsonObject.containsKey("unionid")) {
            unionid = jsonObject.get("unionid").toString();
        }
        String[] arrayRefVar = {openId, unionid};
        return arrayRefVar;
    }

    // 小程序
    public  String[] getOpenid(String code) {
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("appid", appletAppKey);
        paramMap.put("secret", appletAppSecret);
        paramMap.put("js_code", code);
        paramMap.put("grant_type", "authorization_code");
        String result = HttpUtil.get("https://api.weixin.qq.com/sns/jscode2session", paramMap);
        JSONObject jsonObject = JSONObject.parseObject(result);
        if (jsonObject.containsKey("errcode")) {
            //throw new MyException(result);
            return null;
        }
        String openId = jsonObject.get("openid").toString();
        String unionid = "";
        if (jsonObject.containsKey("unionid")) {
            unionid = jsonObject.get("unionid").toString();
        }
        String[] arrayRefVar = {openId, unionid};
        return arrayRefVar;
    }

    //获取通讯录或发消息token
    public String getToken(Integer type) {
        HashMap<String, Object> paramMap = new HashMap<>();
        String url = "https://api.weixin.qq.com/cgi-bin/token";
        if (type == 1) {
            url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";
            paramMap.put("corpsecret", appSecret);
            paramMap.put("corpid", appKey);
        } else if (type == 2) {
            paramMap.put("grant_type", "client_credential");
            paramMap.put("appid", mpAppKey);
            paramMap.put("secret", mpAppSecret);
        }
        String result = HttpUtil.get(url, paramMap);
        JSONObject jsonObject = JSONObject.parseObject(result);
        String token = jsonObject.get("access_token").toString();
        return token;
    }

    //根据所有部门信息
    public List<WeChatDepartDto> getDepartmentList(Long dept_id) {
        String token = this.getToken(1);
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("access_token", token);
        if (dept_id != null) {
            paramMap.put("id", dept_id);
        }
        String result = HttpUtil.get("https://qyapi.weixin.qq.com/cgi-bin/department/list", paramMap);
        JSONObject jsonObject = JSONObject.parseObject(result);
        Object department = jsonObject.get("department");
        List<WeChatDepartDto> weChatDepartDtos = JSONObject.parseArray(department.toString(), WeChatDepartDto.class);
        return weChatDepartDtos;
    }

    //获取部门的所有用户信息
    public List<WeChatUserDto> getDepartmentUser(Long dept_id) {
        String token = this.getToken(1);
            HashMap<String, Object> paramMap = new HashMap<>();
            paramMap.put("access_token", token);
            paramMap.put("department_id", dept_id);
            paramMap.put("fetch_child", 1);
            String result = HttpUtil.get("https://qyapi.weixin.qq.com/cgi-bin/user/list", paramMap);
          JSONObject jsonObject = JSONObject.parseObject(result);
            Object department = jsonObject.get("userlist");
        List<WeChatUserDto> weChatUserDtos = JSONObject.parseArray(department.toString(), WeChatUserDto.class);
        return weChatUserDtos;
    }

    //公众号发送文本消息给用户
    public Boolean sendMessage(String content, String userId) {
        String token = this.getToken(2);
        JSONObject object = new JSONObject();
        JSONArray touser=new JSONArray();
        touser.add(userId);
        object.put("touser", touser);
        object.put("msgtype", "text");
        JSONObject text = new JSONObject();
        text.put("content", content);
        object.put("text",text);
        String result = HttpUtil.post("https://api.weixin.qq.com/cgi-bin/message/mass/send?access_token="+token, JSONObject.toJSONString(object));
        System.out.printf(result);
        return true;
    }

    //{
    //           "touser":"OPENID",
    //           "template_id":"ngqIpbwh8bUfcSsECmogfXcV14J0tQlEpBO27izEYtY",
    //           "url":"http://weixin.qq.com/download",
    //           "miniprogram":{
    //             "appid":"xiaochengxuappid12345",
    //             "pagepath":"index?foo=bar"
    //           },
    //           "client_msg_id":"MSG_000001",
    //           "data":{
    //                   "keyword1":{
    //                       "value":"巧克力"
    //                   },
    //                   "keyword2": {
    //                       "value":"39.8元"
    //                   },
    //                   "keyword3": {
    //                       "value":"2014年9月22日"
    //                   }
    //           }
    //       }
    public Boolean sendTemplateMessage(JSONObject object){
        String token = this.getToken(2);
        String result = HttpUtil.post("https://api.weixin.qq.com/cgi-bin/message/template/send?access_token="+token, JSONObject.toJSONString(object));
        System.out.printf(result);
        return true;
    }
}
