package com.xjrsoft.module.student.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yulichang.base.MPJBaseServiceImpl;
import com.github.yulichang.toolkit.MPJWrappers;
import com.xjrsoft.common.enums.ScoreTypeEnum;
import com.xjrsoft.common.utils.VoToColumnUtil;
import com.xjrsoft.module.base.entity.BaseClass;
import com.xjrsoft.module.base.mapper.BaseClassMapper;
import com.xjrsoft.module.oa.entity.Schedule;
import com.xjrsoft.module.student.dto.BaseStudentAssessmentInspectionMobilePageDto;
import com.xjrsoft.module.student.dto.BaseStudentAssessmentInspectionPageDto;
import com.xjrsoft.module.student.entity.BaseStudentAssessmentClassRelation;
import com.xjrsoft.module.student.entity.BaseStudentAssessmentInspection;
import com.xjrsoft.module.student.entity.BaseStudentAssessmentStudentRelation;
import com.xjrsoft.module.student.entity.BaseStudentCadre;
import com.xjrsoft.module.student.mapper.BaseStudentAssessmentClassRelationMapper;
import com.xjrsoft.module.student.mapper.BaseStudentAssessmentInspectionMapper;
import com.xjrsoft.module.student.mapper.BaseStudentAssessmentStudentRelationMapper;
import com.xjrsoft.module.student.service.IBaseStudentAssessmentInspectionService;
import com.xjrsoft.module.student.vo.*;
import com.xjrsoft.module.system.entity.File;
import com.xjrsoft.module.system.service.IFileService;
import com.xjrsoft.module.teacher.entity.XjrUser;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @title: 学生班级巡查考核
 * @Author dzx
 * @Date: 2023-11-16
 * @Version 1.0
 */
@Service
@AllArgsConstructor
public class BaseStudentAssessmentInspectionServiceImpl extends MPJBaseServiceImpl<BaseStudentAssessmentInspectionMapper, BaseStudentAssessmentInspection> implements IBaseStudentAssessmentInspectionService {

    private final BaseStudentAssessmentInspectionMapper assessmentInspectionMapper;

    private final BaseStudentAssessmentStudentRelationMapper assessmentStudentRelationMapper;

    private final BaseStudentAssessmentClassRelationMapper assessmentClassRelationMapper;

    private final BaseClassMapper baseClassMapper;

    private final IFileService fileService;

    @Override
    public Page<BaseStudentAssessmentInspectionPageVo> getPage(Page<BaseStudentAssessmentInspectionPageDto> page, BaseStudentAssessmentInspectionPageDto dto) {
        Page<BaseStudentAssessmentInspectionPageVo> result = assessmentInspectionMapper.getPage(page, dto);
        // 处理加减分
        result.getRecords().forEach((node) -> {
            if (node.getScoreType().equals(ScoreTypeEnum.ScoreMinus.getCode())) {
                node.setScore(node.getScore() * -1);
            }
        });
        return result;
    }

    @Override
    public BaseStudentAssessmentInspectionVo getInfo(Long id) {
        BaseStudentAssessmentInspectionVo result = assessmentInspectionMapper.getInfo(id);
        // 处理加减分
        if (result.getScoreType().equals(ScoreTypeEnum.ScoreMinus.getCode())) {
            result.setScore(result.getScore() * -1);
        }

        // 获取班级信息
        List<BaseStudentAssessmentClassListVo> baseStudentAssessmentClassListVos = assessmentClassRelationMapper.selectJoinList(BaseStudentAssessmentClassListVo.class,
                MPJWrappers.<BaseStudentAssessmentClassRelation>lambdaJoin()
                        .leftJoin(BaseClass.class, BaseClass::getId, BaseStudentAssessmentClassRelation::getClassId)
                        .leftJoin(XjrUser.class, XjrUser::getId, BaseClass::getTeacherId)
                        .eq(BaseStudentAssessmentClassRelation::getBaseStudentAssessmentInspectionId, id)
                        .select(BaseStudentAssessmentClassRelation::getId)
                        .selectAs(BaseClass::getName, BaseStudentAssessmentClassListVo::getClassName)
                        .selectAs(XjrUser::getName, BaseStudentAssessmentClassListVo::getTeacherName)
                        .selectAs(XjrUser::getMobile, BaseStudentAssessmentClassListVo::getMobile)
                        .select(BaseStudentAssessmentClassRelation.class, x -> VoToColumnUtil.fieldsToColumns(BaseStudentAssessmentClassRelation.class).contains(x.getProperty()))

        );
        result.setClassList(baseStudentAssessmentClassListVos);


        // 获取学生信息
        List<BaseStudentAssessmentStudentRelation> baseStudentAssessmentStudentRelations = assessmentStudentRelationMapper.selectList(
                Wrappers.<BaseStudentAssessmentStudentRelation>query().lambda()
                        .eq(BaseStudentAssessmentStudentRelation::getBaseStudentAssessmentInspectionId, result.getId())
        );
        result.setStudentList(baseStudentAssessmentStudentRelations);

        // 获取文件列表
        result.setFileInfos(fileService.list(Wrappers.<File>query().lambda().eq(File::getFolderId, result.getFileId())));

        return result;
    }

    @Override
    public Page<BaseStudentAssessmentInspectionMobilePageVo> getMobilePage(Page<BaseStudentAssessmentInspectionMobilePageDto> page, BaseStudentAssessmentInspectionMobilePageDto dto) {
        if (dto.getClassIds() == null || dto.getClassIds().size() == 0) {
            // 获取当前用户所管班级
            List<Long> classIds = baseClassMapper.selectList(
                    Wrappers.<BaseClass>query().lambda()
                            .eq(BaseClass::getTeacherId, StpUtil.getLoginIdAsLong())
                            .select(BaseClass::getId)
            ).stream().map(BaseClass::getId).collect(Collectors.toList());
            dto.setClassIds(classIds);
        }
        if (dto.getClassIds() == null || dto.getClassIds().size() == 0) {
            return null;
        }
        Page<BaseStudentAssessmentInspectionMobilePageVo> result = assessmentInspectionMapper.getMobilePage(page, dto);
        // 处理加减分
        result.getRecords().forEach((node) -> {
            if (node.getScoreType().equals(ScoreTypeEnum.ScoreMinus.getCode())) {
                node.setScore(node.getScore() * -1);
            }
        });
        return result;
    }

    @Override
    public BaseStudentAssessmentInspectionMobileVo getMobileInfo(Long id) {
        BaseStudentAssessmentInspectionMobileVo result = assessmentInspectionMapper.getMobileInfo(id);
        // 处理加减分
        if (result.getScoreType().equals(ScoreTypeEnum.ScoreMinus.getCode())) {
            result.setScore(result.getScore() * -1);
        }

        // 获取学生信息
        List<BaseStudentAssessmentStudentRelation> baseStudentAssessmentStudentRelations = assessmentStudentRelationMapper.selectList(
                Wrappers.<BaseStudentAssessmentStudentRelation>query().lambda()
                        .eq(BaseStudentAssessmentStudentRelation::getBaseStudentAssessmentInspectionId, result.getBaseStudentAssessmentInspectionId())
                        .eq(BaseStudentAssessmentStudentRelation::getClassId, result.getClassId())
        );
        result.setStudentList(baseStudentAssessmentStudentRelations);
        result.setStudentCount(baseStudentAssessmentStudentRelations.size());

        // 获取文件列表
        result.setFileInfos(fileService.list(Wrappers.<File>query().lambda().eq(File::getFolderId, result.getFileId())));

        return result;
    }
}
